package czerwony.kapturek.jbspell;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.borland.jbcl.layout.*;


public class SuggestionsDialog extends JDialog
{
    private DialogListener listener;
    private DefaultListModel model = new DefaultListModel();

    private JPanel panel1 = new JPanel();
    private JPanel panel2 = new JPanel();
    private JTextField textWord = new JTextField();
    private JScrollPane scrollSuggestions = new JScrollPane();
    private JList listSuggestions = new JList(model);
    private JLabel labelSuggestions = new JLabel("Suggestions:");
    private JButton buttonChange = new JButton("Change");
    private JButton buttonIgnore = new JButton("Ignore");
    private JButton buttonIgnoreAll = new JButton("Ignore all");
    private JButton buttonAddToDictionary = new JButton("Add to dictionary");
    private JButton buttonClose = new JButton("Close");
    private JButton buttonChangeAll = new JButton("Change all");
    private GridBagLayout gridBagLayout = new GridBagLayout();

    private final static int    STATE_DISABLED      = 0;
    private final static int    STATE_NOSUGGESTIONS = 1;
    private final static int    STATE_HASSUGGESTIONS   = 2;

    private int     state = STATE_DISABLED;
    private boolean changed;


    public SuggestionsDialog(Frame frame, String title, boolean modal)
    {
        super(frame, title, modal);

        initUi();
        pack();
        initListeners();
    }


    public SuggestionsDialog()
    {
        this(null, "", false);
    }


    public SuggestionsDialog(Frame f, DialogListener listener)
    {
        this(f, "", false);

        this.listener = listener;
    }


    void initUi()
    {
        panel1.setLayout(gridBagLayout);

        labelSuggestions.setText("Suggestions:");
        panel1.add(textWord,
                   new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0,
                                          GridBagConstraints.WEST,
                                          GridBagConstraints.HORIZONTAL,
                                          new Insets(14, 11, 0, 0), 256, 6));
        panel1.add(labelSuggestions,
                   new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                          GridBagConstraints.WEST,
                                          GridBagConstraints.NONE,
                                          new Insets(12, 11, 0, 135), 52, 0));
        panel1.add(scrollSuggestions,
                   new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0,
                                          GridBagConstraints.CENTER,
                                          GridBagConstraints.BOTH,
                                          new Insets(0, 11, 12, 0), 0, 38));
        panel1.add(panel2,
                   new GridBagConstraints(1, 0, 1, 3, 1.0, 1.0,
                                          GridBagConstraints.CENTER,
                                          GridBagConstraints.BOTH,
                                          new Insets(14, 19, 12, 3), 19, 64));
        scrollSuggestions.getViewport().add(listSuggestions, null);

        buttonChange.setHorizontalAlignment(SwingConstants.LEFT);
        buttonChange.setMnemonic('C');
        buttonChangeAll.setHorizontalAlignment(SwingConstants.LEFT);
        buttonChangeAll.setMnemonic('A');
        buttonIgnore.setHorizontalAlignment(SwingConstants.LEFT);
        buttonIgnore.setMnemonic('I');
        buttonIgnoreAll.setHorizontalAlignment(SwingConstants.LEFT);
        buttonIgnoreAll.setHorizontalTextPosition(SwingConstants.LEFT);
        buttonIgnoreAll.setMnemonic('G');
        buttonAddToDictionary.setHorizontalAlignment(SwingConstants.LEFT);
        buttonAddToDictionary.setHorizontalTextPosition(SwingConstants.LEFT);
        buttonAddToDictionary.setMnemonic('D');
        buttonClose.setHorizontalAlignment(SwingConstants.LEFT);
        Font fontTmp = textWord.getFont();
        textWord.setFont(new Font(fontTmp.getName(),
                                  fontTmp.getStyle() | Font.BOLD,
                                  fontTmp.getSize()));

        JPanel panelTop =
            new JPanel(new VerticalFlowLayout(VerticalFlowLayout.TOP, 0, 0,
                                              true, false));
        panelTop.add(buttonChange);
        panelTop.add(buttonChangeAll);
        panelTop.add(buttonIgnore);
        panelTop.add(buttonIgnoreAll);
        panelTop.add(buttonAddToDictionary);

        JPanel panelBottom =
            new JPanel(new VerticalFlowLayout(VerticalFlowLayout.BOTTOM, 0, 0,
                                              true, false));
        panelBottom.add(buttonClose);
        panel2.setLayout(new BorderLayout());
        panel2.add(panelTop, BorderLayout.NORTH);
        panel2.add(panelBottom, BorderLayout.SOUTH);

        getContentPane().add(panel1);
        this.setTitle("Spell checker");
        state = STATE_DISABLED;
        adjustButtons();
    }


    private void adjustButtons()
    {
        buttonChange.setEnabled(changed);
        buttonChangeAll.setEnabled(changed);
        buttonAddToDictionary.setEnabled(state != STATE_DISABLED);
        buttonIgnore.setEnabled(state != STATE_DISABLED);
        buttonIgnoreAll.setEnabled(state != STATE_DISABLED);
    }


    private void initListeners()
    {
        addKeyListener(new KeyAdapter()
            {
                public void keyPressed(KeyEvent ev)
                {
                    try
                    {
                        if (ev.getKeyCode() == KeyEvent.VK_ESCAPE)
                            listener.onAbortChecking();
                    }
                    catch (SpellCheckerException ex)
                    {
                    }
                }
            });

        buttonClose.addActionListener(new DlgActionListener()
            {
                public void onAction(ActionEvent e)
                    throws SpellCheckerException
                {
                    listener.onAbortChecking();
                }
            });

        buttonIgnore.addActionListener(new DlgActionListener()
            {
                public void onAction(ActionEvent e)
                    throws SpellCheckerException
                {
                    state = STATE_DISABLED;
                    adjustButtons();
                    listener.onIgnore("");
                }
            });

        buttonIgnoreAll.addActionListener(new DlgActionListener()
            {
                public void onAction(ActionEvent e)
                    throws SpellCheckerException
                {
                    state = STATE_DISABLED;
                    adjustButtons();
                    listener.onIgnoreAll(textWord.getText());
                }
            });

        buttonAddToDictionary.addActionListener(new DlgActionListener()
            {
                public void onAction(ActionEvent e)
                    throws SpellCheckerException
                {
                    state = STATE_DISABLED;
                    adjustButtons();
                    listener.onAddToDictionary(textWord.getText());
                }
            });

        buttonChange.addActionListener(new DlgActionListener()
            {
                public void onAction(ActionEvent e)
                    throws SpellCheckerException
                {
                    state = STATE_DISABLED;
                    adjustButtons();
                    listener.onReplace(textWord.getText(), listSuggestions.getSelectedValue().toString());
                }
            });

        buttonChangeAll.addActionListener(new DlgActionListener()
            {
                public void onAction(ActionEvent e)
                    throws SpellCheckerException
                {
                    state = STATE_DISABLED;
                    adjustButtons();
                    listener.onReplaceAll(textWord.getText(), listSuggestions.getSelectedValue().toString());
                }
            });

        listSuggestions.addListSelectionListener(new ListSelectionListener()
            {
                public void valueChanged(ListSelectionEvent ev)
                {
                    textWord.setText(listSuggestions.getSelectedValue().toString());
                    changed = true;
                    adjustButtons();
                }
            });

        textWord.addKeyListener(new KeyAdapter()
            {
                public void keyTyped(KeyEvent ev)
                {
                    if (Character.isDefined(ev.getKeyChar()))
                    {
                        changed = true;
                        adjustButtons();
                    }
                }
            });
    }


    void setSuggestions(String word, String[] suggestions)
    {
        model.clear();
        textWord.setText(word);
        changed = false;

        if (suggestions == null)
        {
            state = STATE_NOSUGGESTIONS;
            adjustButtons();
            return;
        }

        for (int i = 0; i < suggestions.length; i++)
        {
            model.addElement(suggestions[i]);
        }

        state = suggestions.length > 0 ? STATE_HASSUGGESTIONS : STATE_NOSUGGESTIONS;
        adjustButtons();
    }


    private abstract class DlgActionListener implements java.awt.event.ActionListener
    {
        public void actionPerformed(ActionEvent ev)
        {
            try
            {
                onAction(ev);
            }
            catch (SpellCheckerException ex)
            {
                ex.printStackTrace();
            }
        }

        protected abstract void onAction(ActionEvent ev) throws SpellCheckerException;
    }


    static interface DialogListener
    {
        public void onReplace(String oldWord, String newWord)       throws SpellCheckerException;
        public void onReplaceAll(String oldWord, String newWord)    throws SpellCheckerException;
        public void onIgnore(String word)                           throws SpellCheckerException;
        public void onIgnoreAll(String word)                        throws SpellCheckerException;
        public void onAddToDictionary(String word)                  throws SpellCheckerException;
        public void onAbortChecking()                               throws SpellCheckerException;
    }

}


